﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Win_Flexgrid_Undo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        List<DataList> list_z, list_y;        

        private void Form1_Load(object sender, EventArgs e)
        {

            this.c1FlexGrid1.Cols.Count = 5;
            this.c1FlexGrid1.Rows.Count = 10;

            #region "add data"
            for (int _row = this.c1FlexGrid1.Rows.Fixed; _row < this.c1FlexGrid1.Rows.Count; _row++)
            {
                for (int _col = this.c1FlexGrid1.Rows.Fixed; _col < this.c1FlexGrid1.Cols.Count; _col++)
                {
                    this.c1FlexGrid1[_row, _col] = string.Format("{0},{1}", _row, _col);
                }
            }
            #endregion

            list_z = new List<DataList>();
            list_y = new List<DataList>();
            //let the grid handle clipboard operations
            this.c1FlexGrid1.AutoClipboard = true;

            #region "event handlers"
            this.c1FlexGrid1.KeyUp += new KeyEventHandler(c1FlexGrid1_KeyUp);
            this.c1FlexGrid1.ValidateEdit += new C1.Win.C1FlexGrid.ValidateEditEventHandler(c1FlexGrid1_ValidateEdit);
            this.c1FlexGrid1.KeyDown += new KeyEventHandler(c1FlexGrid1_KeyDown);
            #endregion
        }

        //Save original cell values to list before a new value gets pasted
        void c1FlexGrid1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == (Keys.Control | Keys.V))
            {
                int row = this.c1FlexGrid1.Row;
                int col = this.c1FlexGrid1.Col;
                string value = this.c1FlexGrid1.GetData(row, col).ToString();
                list_z.Add(new DataList() { Row = row, Col = col, Text = value });
            }
        }

        //Add values to "list" object as grid's contents get changed.
        void c1FlexGrid1_ValidateEdit(object sender, C1.Win.C1FlexGrid.ValidateEditEventArgs e)
        {
            list_z.Add(new DataList() { Row = e.Row, Col = e.Col, Text = this.c1FlexGrid1[e.Row, e.Col].ToString() });
        }

        void c1FlexGrid1_KeyUp(object sender, KeyEventArgs e)
        {
            //Undo changes
            if ((e.KeyData == (Keys.Control | Keys.Z) && list_z.Count != 0))
            {
                //Iterate the list object from bottom to top; and fetch row,col and text
                //save the same value to the same row,col of the grid
                int row = list_z[list_z.Count - 1].Row;
                int col = list_z[list_z.Count - 1].Col;
                string value = list_z[list_z.Count - 1].Text;

                //Add new cell values to Redo list
                list_y.Add(new DataList() { Row = row, Col = col, Text = this.c1FlexGrid1[row, col].ToString() });

                //Save original cell value to the grid and set focus to the cell                
                this.c1FlexGrid1[row, col] = value;
                this.c1FlexGrid1.Row = row;
                this.c1FlexGrid1.Col = col;
                //remove last element from the list
                list_z.RemoveAt(list_z.Count - 1);
            }

            //Redo changes
            else if ((e.KeyData == (Keys.Control | Keys.Y) && list_y.Count != 0))
            {
                int row = list_y[list_y.Count - 1].Row;
                int col = list_y[list_y.Count - 1].Col;
                string value = list_y[list_y.Count - 1].Text;

                //Add existing values to Undo list
                list_z.Add(new DataList() { Row = row, Col = col, Text = this.c1FlexGrid1[row, col].ToString() });

                //Save original cell value to the grid and set focus to the cell                
                this.c1FlexGrid1[row, col] = value;
                this.c1FlexGrid1.Row = row;
                this.c1FlexGrid1.Col = col;
                //remove last element from the list
                list_y.RemoveAt(list_y.Count - 1);
            }
        }
    }

    #region "Class"
    public class DataList
    {
        private int _row;
        private int _col;
        private string _text;

        public int Row
        {
            get { return _row; }
            set { _row = value; }
        }

        public int Col
        {
            get { return _col; }
            set { _col = value; }
        }

        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }
    }
    #endregion
}
